@@ -1,67 +1,27 @@
 package Error::TypeTiny;
 
-use 5.006001;
+use 5.008001;
 use strict;
 use warnings;
 
 BEGIN {
 	$Error::TypeTiny::AUTHORITY = 'cpan:TOBYINK';
-	$Error::TypeTiny::VERSION   = '1.012002';
+	$Error::TypeTiny::VERSION   = '2.010001';
 }
 
 $Error::TypeTiny::VERSION =~ tr/_//d;
 
 require Type::Tiny;
 __PACKAGE__->Type::Tiny::_install_overloads(
-	q[""]   => sub { $_[0]->to_string },
+	q[""]   => sub { local $@; $_[0]->to_string },
 	q[bool] => sub { 1 },
 );
 
+require Carp;
+*CarpInternal = \%Carp::CarpInternal;
+
 our %CarpInternal;
-$CarpInternal{$_}++ for qw(
-	Types::Standard::_Stringable
-	Exporter::Tiny
-	Eval::TypeTiny::Sandbox
-	
-	Devel::TypeTiny::Perl56Compat
-	Devel::TypeTiny::Perl58Compat
-	Error::TypeTiny
-	Error::TypeTiny::Assertion
-	Error::TypeTiny::Compilation
-	Error::TypeTiny::WrongNumberOfParameters
-	Eval::TypeTiny
-	Reply::Plugin::TypeTiny
-	Test::TypeTiny
-	Type::Coercion
-	Type::Coercion::FromMoose
-	Type::Coercion::Union
-	Type::Library
-	Type::Params
-	Type::Parser
-	Type::Registry
-	Types::Common::Numeric
-	Types::Common::String
-	Types::Standard
-	Types::Standard::ArrayRef
-	Types::Standard::CycleTuple
-	Types::Standard::Dict
-	Types::Standard::HashRef
-	Types::Standard::Map
-	Types::Standard::ScalarRef
-	Types::Standard::StrMatch
-	Types::Standard::Tied
-	Types::Standard::Tuple
-	Types::TypeTiny
-	Type::Tiny
-	Type::Tiny::Class
-	Type::Tiny::Duck
-	Type::Tiny::Enum
-	Type::Tiny::_HalfOp
-	Type::Tiny::Intersection
-	Type::Tiny::Role
-	Type::Tiny::Union
-	Type::Utils
-);
+$CarpInternal{$_}++ for @Type::Tiny::InternalPackages;
 
 sub new {
 	my $class  = shift;
@@ -70,7 +30,14 @@ sub new {
 }
 
 sub throw {
+	my $next = $_[0]->can( 'throw_cb' );
+	splice( @_, 1, 0, undef );
+	goto $next;
+}
+
+sub throw_cb {
 	my $class = shift;
+	my $callback = shift;
 	
 	my ( $level, @caller, %ctxt ) = 0;
 	while (
@@ -89,14 +56,17 @@ sub throw {
 		$level++ if caller( $level ) eq ( $pkg || "" );
 	}
 	
-	# Moo's Method::Generate::Constructor puts an eval in the stack trace,
-	# that is useless for debugging, so show the stack frame one above.
-	$level++
-		if (
-		( caller( $level ) )[1] =~ /^\(eval \d+\)$/
-		and ( caller( $level ) )[3] eq '(eval)'    # (caller())[3] is $subroutine
-		);
-	@ctxt{qw/ package file line /} = caller( $level );
+	{
+		no warnings 'uninitialized';
+		# Moo's Method::Generate::Constructor puts an eval in the stack trace,
+		# that is useless for debugging, so show the stack frame one above.
+		$level++
+			if (
+			( caller( $level ) )[1] =~ /^\(eval \d+\)$/
+			and ( caller( $level ) )[3] eq '(eval)'    # (caller())[3] is $subroutine
+			);
+		@ctxt{qw/ package file line /} = caller( $level );
+	}
 	
 	my $stack = undef;
 	if ( our $StackTrace ) {
@@ -106,13 +76,13 @@ sub throw {
 		);
 	}
 	
-	die(
-		our $LastError = $class->new(
-			context     => \%ctxt,
-			stack_trace => $stack,
-			@_,
-		)
+	our $LastError = $class->new(
+		context     => \%ctxt,
+		stack_trace => $stack,
+		@_,
 	);
+	
+	$callback ? $callback->( $LastError ) : die( $LastError );
 } #/ sub throw
 
 sub message     { $_[0]{message} ||= $_[0]->_build_message }
@@ -123,7 +93,7 @@ sub to_string {
 	my $e = shift;
 	my $c = $e->context;
 	my $m = $e->message;
-	
+
 	$m =~ /\n\z/s
 		? $m
 		: $c ? sprintf(
@@ -160,19 +130,18 @@ Error::TypeTiny - exceptions for Type::Tiny and friends
 
 =head1 SYNOPSIS
 
+   use feature 'try';
    use Data::Dumper;
-   use Try::Tiny;
-   use Types::Standard qw(Str);
+   use Types::Standard qw( Str );
    
    try {
-      Str->assert_valid(undef);
+      Str->assert_valid( undef );
    }
-   catch {
-      my $exception = shift;
+   catch ( $exception ) {
       warn "Encountered Error: $exception";
-      warn Dumper($exception->explain)
-         if $exception->isa("Error::TypeTiny::Assertion");
-   };
+      warn Dumper( $exception->explain )
+         if $exception->isa( "Error::TypeTiny::Assertion" );
+   }
 
 =head1 STATUS
 
@@ -198,6 +167,13 @@ Constructs an exception and passes it to C<die>.
 
 Automatically populates C<context> and C<stack_trace> if appropriate.
 
+=item C<< throw_cb($callback, %attributes) >>
+
+Constructs an exception and passes it to C<< $callback >> which should
+be a coderef; if undef, uses C<die>.
+
+Automatically populates C<context> and C<stack_trace> if appropriate.
+
 =back
 
 =head2 Attributes
@@ -254,9 +230,10 @@ Stringification is overloaded to call C<to_string>.
 
 =over
 
-=item C<< %Error::TypeTiny::CarpInternal >>
+=item C<< %Carp::CarpInternal >>
 
-Serves a similar purpose to C<< %Carp::CarpInternal >>.
+Error::TypeTiny honours this package variable from L<Carp>.
+(C< %Error::TypeTiny::CarpInternal> is an alias for it.)
 
 =item C<< $Error::TypeTiny::StackTrace >>
 
@@ -298,7 +275,7 @@ Toby Inkster E<lt>tobyink@cpan.orgE<gt>.
 
 =head1 COPYRIGHT AND LICENCE
 
-This software is copyright (c) 2013-2014, 2017-2021 by Toby Inkster.
+This software is copyright (c) 2013-2014, 2017-2025 by Toby Inkster.
 
 This is free software; you can redistribute it and/or modify it under
 the same terms as the Perl 5 programming language system itself.
